// thornberry/patientActivity/ActivityEditor.js
define([
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./templates/ActivityEditor.html",
	"dojo/date/locale",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/store/Memory",
	"dojo/store/Observable",
	"dojo/data/ObjectStore",
	"dojo/query",
	"dojo/Deferred",
	"dijit/form/FilteringSelect",
	"dijit/form/Select",
	"dijit/form/DateTextBox",
	"thornberry/form/TimeTextBox",
	"dijit/form/ValidationTextBox",
	"dijit/form/ComboBox",
	"dijit/form/Textarea",
	"dijit/form/CheckBox",
	"dijit/form/Form",
	"dijit/form/DropDownButton",
	"dijit/form/TextBox",
	"dijit/TooltipDialog",
	"dijit/Tooltip",
	"dijit/form/Button",
	"dijit/popup",
	"dijit/registry",
	"thornberry/patientActivity/Activity",
	"thornberry/object/TableManager",
	"thornberry/serverRequest",
	"dojox/form/CheckedMultiSelect",
	"dojo/on",
	"dojo/date",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/data/ItemFileReadStore",
	"dojo/promise/all",
	"dojo/when",
	"thornberry/editor/property/StatusTag",
	"thornberry/schedule/ConflictChecker",
	"dojo/NodeList-dom",
	"dojox/html/entities"	// used by escapeText(), preload here to make sure it's usable
], function(declare, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, template, locale, domStyle, domClass, domConstruct, 
			domGeom, Memory, Observable, ObjectStore, query, Deferred, FilteringSelect, Select, DateTextBox, TimeTextBox, ValidationTextBox, 
			ComboBox, Textarea, CheckBox, Form, DropDownButton, TextBox, TooltipDialog, Tooltip, Button, popup, registry, Activity, 
			TableManager, serverRequest, CheckedMultiSelect, on, date, lang, arrayUtils, ItemStore, all, when, StatusTag, ConflictChecker) {
        var ActivityEditor = declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
			templateString: template,
			baseClass: "ActivityEditor",
			
			modified: false,
			activity: null,
			activityCopy: null,
			actionType: "",
			requiredFields: null,
			
			postMixInProperties: function() {
				var t = this;
				t.inherited(arguments);
				
				t.activityNew = lang.clone(t.activity);
				t.requiredFields = {};
			},
			
			buildRendering: function() {
				var t = this;
				t.inherited(arguments);
				
				var activity = t.activityNew;
				var disabled = (t.actionType === "View" || t.actionType === "Prepare" ? true : false);
				
				t.titleNode.innerHTML = t.actionType + " " + activityTypeStore.get(activity.typeId).name;
				
				t.dateField = new DateTextBox({
					style: "width:110px",
					disabled: disabled,
					value: activity.date
				},t.dateNode);
				t.dateField.startup();
				t.own(on(t.dateField,"change",function(value) {
					if (dischargeDate && date.difference(dischargeDate,value,"day") > 0) {
						actionShowMessage("Warning: this date is after the patient's discharge date.");
					}
					t._modifyValue("date");
					t.startTimeField.validate();
					t.endTimeField.validate();
				}));
				t.own(on(t.dateField,"blur",function(){checkForT(t.dateField.id);}));
				t.dateField.set("constraints",{min:date.add(new Date(),"day",-3700)});
				t.dateField.set("constraints",{max:date.add(new Date(),"day",365)});
				t.require(t.dateField);
				
				t.startTimeField = new TimeTextBox({
					constraints: {timePattern: "HHmm", clickableIncrement: "T00:15:00", visibleIncrement: "T01:00:00", visibleRange: "T01:00:00"},
					style: "width:90px",
					maxHeight: -1,
					disabled: disabled,
					value: activity.startTime
				},t.startTimeNode);
				t.startTimeField.startup();
				t.own(on(t.startTimeField,"change",function() {t._modifyValue("startTime");}));
				t.own(on(t.startTimeField,"blur",function(){checkForN(t.startTimeField.id);}));
				t.require(t.startTimeField);
				
				t.endTimeField = new TimeTextBox({
					constraints: {timePattern: "HHmm", clickableIncrement: "T00:15:00", visibleIncrement: "T01:00:00", visibleRange: "T01:00:00"},
					style: "width:90px",
					maxHeight: -1,
					disabled: disabled,
					value: activity.endTime
				},t.endTimeNode);
				t.endTimeField.startup();
				t.own(on(t.endTimeField,"change",function() {t._modifyValue("endTime");}));
				t.own(on(t.endTimeField,"blur",function(){checkForN(t.endTimeField.id);}));
				
				t.totalTimeField = new ValidationTextBox({
					style: "width:5em",
					validator: function(value) {
						var valid = true;
						if (this.required || value !== "") {
							var regExp = /^(\d{1,3}:\d{2}|\d+)$/;
							valid = regExp.test(value);
							if (valid) {
								var pieces = value.toString().split(":");
								if (pieces.length > 1 && parseInt(pieces[1]) >= 60) {
									valid = false;
								}
							}
						}
						return valid;
					},
					invalidMessage: "Total time must be entered as H:MM",
					disabled: disabled,
					value: activity.totalTime
				},t.totalTimeNode);
				t.totalTimeField.startup();
				t.own(on(t.totalTimeField,"change",lang.hitch(t,t._modifyTotalTime)));
				
				// Filter reasons by activity type
				var restrictReasonArray = [];
				arrayUtils.forEach(reasonStore.data,function(reason) {
					if (arrayUtils.indexOf(reason.activityTypeList.toString().split(","),activity.typeId) !== -1) {
						restrictReasonArray.push(reason.id);
					}
				});
				
				t.reasonIdField = new Select({
					disabled: disabled,
					required: false,
					style: "width:200px"
				},t.reasonNode);
				t.reasonIdField.startup();
				t.own(on(t.reasonIdField,"change",lang.hitch(t,t._modifyReasonId)));
				t.populateSelect(t.reasonIdField,reasonStore,activity.reasonId,(t.actionType !== "Add"),restrictReasonArray);
				t.require(t.reasonIdField);
				
				var reason = reasonStore.get(activity.reasonId);
				t.reasonOtherField = new TextBox({
					maxLength: 30,
					disabled: (disabled || !(reason && reason.canEdit == 0 && reason.name === "Other")),
					value: activity.reasonOther
				},t.reasonOtherNode);
				t.reasonOtherField.startup();
				t.own(on(t.reasonOtherField,"change",function() {t._modifyValue("reasonOther");}));
				
				var options = [];
				var otherOption = null;
				arrayUtils.forEach(actionTakenStore.data, function(item) {
					if (item.active === "Yes" && arrayUtils.some(item.activities.toString().split(","),function(activityId) {return activityId == activity.typeId;})) {
						var option = document.createElement("option");
						option.value = item.id;
						option.label = escapeText(item.name.toString());
						(item.id === 7 ? otherOption = option : options.push(option));
					}
				});
				if (otherOption) {
					domStyle.set(otherOption,"display","block");
					options.push(otherOption);
				}
				
				t.actionArrayField = new CheckedMultiSelect({
					options: options,
					multiple: true,
					value: activity.actionArray
				},t.actionTakenNode);
				t.actionArrayField.startup();
				t.own(on(t.actionArrayField,"change",lang.hitch(t,t._modifyActionArray)));
				t.actionArrayField.set("disabled",disabled);
				
				if (t.actionArrayField.wrapperDiv.lastChild) {
					domStyle.set(t.actionArrayField.wrapperDiv.lastChild,"display","block");
					domStyle.set(t.actionArrayField.wrapperDiv.lastChild,"width","500px");
					
					var otherStyle = "margin-left: 20px";

					if(NOT_IE){
						otherStyle = ""
					}

					t.actionOtherField = new TextBox({
						maxLength: 30,
						style: otherStyle,
						disabled: disabled || arrayUtils.every(activity.actionArray,function(actionId) {return actionId !== "7";}),
						value: activity.actionOther
					});
					
					if(NOT_IE){
						domConstruct.place(t.actionOtherField.domNode,t.actionArrayField.domNode);
					}else{
						domConstruct.place(t.actionOtherField.domNode,t.actionArrayField.wrapperDiv.lastChild);
					}
					
					t.actionOtherField.startup();
					t.own(on(t.actionOtherField,"change",function() {t._modifyValue("actionOther");}));
				}
				
				t.summaryField = new Textarea({
					maxLength: 10000,
					disabled: disabled,
					value: activity.summary
				},t.summaryNode);
				t.summaryField.startup();
				t.own(on(t.summaryField,"change",function() {t._modifyValue("summary");}));
				if (requireSummary == 1) {
					t.require(t.summaryField);
				}
				
				activity.buildCustomRendering(t);

				if (conflictCheckingEnabled && (activity.typeId == 1 || activity.typeId == 2 || activity.typeId == 3 || activity.typeId == 5)) {
					t.conflictsField = new StatusTag({},t.ConflictsNode);
					t.conflictsField.startup();
					t.conflictsField.deserialize({statusInt:1,statusText:"Checking..."});
				} else {
					domStyle.set(t.ConflictsContainer,"display","none");
				}
				
				if (!disabled) {
					t.updateFieldPinking();
				}
				t.set("modified",false);
			},
			
			postCreate: function() {
				var t = this;
				t.inherited(arguments);
				
				t.own(t.activityNew.watch(lang.hitch(t,"notify")));

				t.defer("checkConflicts");
			},
			
			okToSave: function() {
				var t = this;
				var ok = false;
				
				if (!t.formNode.validate()) {
					actionShowMessage("Please correct any invalid fields.");
				} else if (query(".pinkBackground",t.containerNode).length > 0) {
					actionShowMessage("Please fill in all required fields.");
				} else if ((t.actionType === "Edit" || t.actionType === "Prepare") && !t.modified) {
					actionShowMessage("You have not made any changes.");
				} else {
					ok = true;
				}
				return ok;
			},
			
			notify: function(name,oldValue,newValue) {
				var t = this;
				var widget = t[name+"Field"];
				if (widget) {
					widget._onChangeActive = false;
					widget.set("value",newValue);
					widget._onChangeActive = true;
				}
			},
			
			_setModifiedAttr: function(value) {
				this.modified = value;
			},
			
			_modifyValue: function(fieldName) {
				var t = this;
				var widget = t[fieldName+"Field"];
				if ((typeof widget.isValid === "undefined") || widget.isValid()) {
					var value = widget.get("value");
					if (typeof value === "string") {
						value = value.toString().replace(/~/g,"");
					}
					t.activityNew.set(fieldName,value);
				}
				t._postModify();
				if (fieldName === "date" || fieldName === "startTime" || fieldName === "endTime") {
					t.defer("checkConflicts");
				}
			},
			
			_modifyTotalTime: function(value) {
				var t = this;
				if (t.totalTimeField.isValid()) {
					value = (value.toString().indexOf(":") === -1 ? Activity.formatMinutes(value) : value);
					t.activityNew.set("totalTime",value);
				}
				t._postModify();
				t.defer("checkConflicts");
			},
			
			_modifyReasonId: function(value) {
				var t = this;
				if (t.reasonIdField.isValid()) {
					t.activityNew.set("reasonId",value);
					var reason = reasonStore.get(value);
					if (reason && reason.canEdit == 0 && reason.name === "Other") {
						t.reasonOtherField.set("disabled",false);
						t.require(t.reasonOtherField);
					} else {
						t.reasonOtherField.set("disabled",true);
						t.require(t.reasonOtherField,false);
						t.activityNew.set("reasonOther","");
					}
				}
				t._postModify();
			},
			
			_modifyActionArray: function(value) {
				var t = this;
				t.activityNew.set("actionArray",value);
				
				if (t.actionOtherField) {
					if (arrayUtils.some(value,function(actionId) {
						return actionId === "7";
					})) {
						t.actionOtherField.set("disabled",false);
						t.require(t.actionOtherField);
					} else {
						t.actionOtherField.set("disabled",true);
						t.require(t.actionOtherField,false);
						t.activityNew.set("actionOther","");
					}
				}
				
				t._postModify();
			},
			
			_postModify: function() {
				var t = this;
				if (!t.activityNew._inLoad) {
					t.set("modified",true);
				}
				t.updateFieldPinking();
			},
			
			require: function(widget,required) {
				if (typeof required === "undefined") {
					required = true;
				} 
				this.requiredFields[widget.id] = required;
			},
			
			updateFieldPinking: function() {
				var t = this;
				var pinkifyField = function(widget,override) {
					var value = widget.get("value");
					if (t.requiredFields[widget.id] && !widget.get("disabled") && (!value || (typeof value === "object" && value.length === 0))) {
						domClass.add(widget.domNode,"pinkBackground");
					} else {
						domClass.remove(widget.domNode,"pinkBackground");
					}
				};
				
				//arrayUtils.forEach(/*t.formNode.getChildren()*/t.requiredFields, function(widgetId) {
				for (var widgetId in t.requiredFields) {
					var widget = registry.byId(widgetId);
					if (widget) {
						pinkifyField(widget);
					}
				}
			},
			
			populateSelect: function(selectWidget,store,value,noDefault,restrictIdArray) {
				var restrictIdArray = restrictIdArray || null;
				var optionArray = [{id:0,name:"(Select)"}];
				var found = false;
				arrayUtils.forEach(store.data,function(item) {
					if (item.active === "Yes" && (restrictIdArray === null || arrayUtils.indexOf(restrictIdArray,item.id) !== -1)) {
						optionArray.push(item);
						if (value && value == item.id) {
							found = true;
						}
					}
				});
				if (!found && value) {
					var item = store.get(value);
					if (item) {
						var item = lang.clone(item);
						item.name += "*";
						optionArray.push(item);
					}
				}
				var optionStore = new Memory({
					data: optionArray,
					getLabel: function(item) {return item.name;}
				});
				selectWidget.setStore(optionStore);
				
				if (value) {
					selectWidget._onChangeActive = false;
					selectWidget.set("value",value);
					selectWidget._onChangeActive = true;
				} else if (selectWidget.options.length === 2 && !noDefault) {
					selectWidget.set("value",selectWidget.options[1].value);
				}
			},
			
			timeValidator: function(value,constraints,editor) {
				var valid = DateTextBox.prototype.validator.call(this,value,constraints);
				if (valid && value && editor.activityNew.date && date.compare(editor.activityNew.date,new Date(),"date") === 0) {
					var time = locale.parse(value,{selector:"time",timePattern:"HHmm"});
					var newTime = new Date();
					newTime.setMinutes(time.getMinutes());
					newTime.setHours(time.getHours());
					if (newTime.getTime() > (new Date()).getTime()) {
						valid = false;
						this.set('invalidMessage','The activity time must not be in the future.');
					} else {
						this.set('invalidMessage','The value entered is not valid.');
					}
				}
				return valid;
			},
			
			close: function() {
				var t = this;

			},
			
			resize: function() {
				var t = this;
				t.inherited(arguments);
				var containerBox = domGeom.getContentBox(t.containerNode);
				var titleBox = domGeom.getContentBox(t.titleNode);
				var height = containerBox.h - titleBox.h - 12;
				if (height < 0) {
					height = 0;
				}
				domStyle.set(t.scrollableDiv,"height",height + "px");
			},

			executeSearch: function(searchString,searchType,resultsDisplayNode) {
				for(var i = resultsDisplayNode.options.length - 1; i >= 0; i--){
					resultsDisplayNode.remove(i);
				}
			
				var fromToItem = fromToStore.get(searchType);
				if (searchString !== "" && fromToItem) {
					var searchPromise;
					if (fromToItem.name === "Agency Employee") {
						searchPromise = this.searchUsers(searchString);
					} else if (fromToItem.name === "Physician") {
						searchPromise = serverRequest("Patient.Activity.Utility", "SearchPhysicians", {"queryString": searchString});
					}
					when(searchPromise,function(results) {
						arrayUtils.forEach(results,function(result) {
							var option = document.createElement("option");
							option.value = result.id;
							
							
							option.innerHTML = (result.displayName || result.name)+(fromToItem.name === "Agency Employee" ? "" : " ("+result.id+")");
							option.name = result.name;
							resultsDisplayNode.appendChild(option);
						});
					});
				}
			},
			
			getUserTeams: function(userId) {
				return serverRequest("Patient.Activity.Utility", "GetUserTeams", {"userId": userId});
			},
			
			searchUsers: function(queryString) {
				var firstNameMatch = new RegExp(","+queryString,"i");
				var lastNameMatch = new RegExp("^"+queryString,"i");
				var userIdMatch = lastNameMatch;
				
				var results = [];
				
				for (var i = 0; i < userStore.data.length; i++) {
					var user = userStore.data[i];
					if (user.active == 1 && (firstNameMatch.test(user.name) || lastNameMatch.test(user.name) || userIdMatch.test(user.id))) {
						results.push(user);
					}
				}
				return results;
			},

			save: function() {
				var saveData = this.activityNew.getSaveData();

				return serverRequest("Patient.Activity.Utility", "SaveActivity", {
					action: this.actionType,
					userId: userId,
					deviceId: deviceId,
					activity: saveData
				});
			},

			_checkStarted: false,

			checkConflicts: function() {
				var t = this;
				var activity = t.activityNew;
				if (t.conflictsField && !t._checkStarted) {
					if (activity.dateString !== "" && activity.startTimeString !== "") {
						t._checkStarted = true;
						t.conflictsField.deserialize({statusInt:1,statusText:"Checking..."});
						setTimeout(function() {
							t.normalizeCalendars(activity.getCalendars()).then(function(calendars){
								var events = [];
								if (typeof activity.careMeetingId !== "undefined") {
									// Special case for patient care meetings - us the meeting ID as the guid for employees
									var employeeCalendars = [];
									calendars = arrayUtils.filter(calendars,function(calendar) {
										var pieces = calendar.toString().split("||");
										if (pieces[1] === "Employee") {
											employeeCalendars.push(calendar);
										}
										return pieces[1] !== "Employee";
									});

									events.push({
										id: "PatientCareMeeting||"+activity.careMeetingId,
										_allDay: (activity.startTimeString === ""),
										division: "",
										_startTime: locale.parse(activity.dateString+" "+(activity.startTimeString !== "" ? activity.startTimeString : "0000"),{selector:"date",datePattern:"yyyy-MM-dd HHmm"}),
										duration: Activity.parseMinutes(activity.totalTime),
										calendars: employeeCalendars,
										travelMinutes: "",
										modifiedStamp: 9999999999
									});
								}

								events.push({
									id: activity.guid,
									_allDay: (activity.startTimeString === ""),
									division: "",
									_startTime: locale.parse(activity.dateString+" "+(activity.startTimeString !== "" ? activity.startTimeString : "0000"),{selector:"date",datePattern:"yyyy-MM-dd HHmm"}),
									duration: Activity.parseMinutes(activity.totalTime),
									calendars: calendars,
									travelMinutes: "",
									modifiedStamp: 9999999999
								});
								
								var checker = new ConflictChecker();
								checker.checkEvents(events).then(function(conflicts) {
									t._checkStarted = false;
									var data;
									if (conflicts.length > 0) {
										data = {
											statusInt: 	2,
											statusText: conflicts.length,
											statusDescription: checker.display(conflicts,false)
										}	
									} else {
										data = {
											statusInt: 	6,
											statusText: "None"
										}	
									}
									t.conflictsField.deserialize(data);
								})
							});
						},150);
					} else {
						t.conflictsField.deserialize({"statusInt":9,"statusText":"Data required"});
					}
				}
			},

			normalizeCalendars: function(calendarArray) {
				// Change userIds to employeeIds
				var deferred = new Deferred();
				var users = [];
				var normalized = arrayUtils.filter(calendarArray,function(calendar) {
					var pieces = calendar.toString().split("||");
					if (pieces[1] === "User") {
						users.push(pieces[0]);
					}
					return pieces[1] !== "User";
				});
				if (users.length === 0) {
					deferred.resolve(normalized);
				} else {
					// Get employee IDs
					serverRequest("UI.Utility","GetEmployeeIds",users).then(function(employeeIds){
						arrayUtils.forEach(employeeIds,function(employeeId) {
							normalized.push(employeeId+"||Employee");
						});
						deferred.resolve(normalized);
					});
				}
				return deferred.promise;
			}
		});
		
		// For use outside of Contact Activity grid. Does not check locks, etc. 
		ActivityEditor.loadActivity = function(activityId, actionType, activityTypeId) {
			var pieces = activityId.split("||");
			ndocNum = pieces[0];
			var counter = pieces[1];

			var deferred = new Deferred();

			this.loadStores(ndocNum, counter).then(function() {
				var activity;
				if (actionType === "Add") {
					activity = Activity.getActivity({typeId:activityTypeId.toString(),ndocNum:ndocNum});
				} else {
					activity = activityStore.get(activityId);
				}

				var activityEditor = new ActivityEditor({
					id: "activityEditor",
					actionType: actionType,
					activity: activity
				});
				deferred.resolve(activityEditor);
			}, function() {
				deferred.reject();
			});
			return deferred.promise;
		};

		// The necessary stores won't exist if the editor is invoked outside the Contact Activity grid
		ActivityEditor.loadStores =  function(ndocNum, counter) {
			var activityList;
			var activitiesPromise = serverRequest("Patient.Activity.Utility", "GetSingleActivity", {"ndocNum": ndocNum, "counter": counter}).then(function(results) {
				activityList = results;
			});

			var statesPromise = serverRequest("Patient.Activity.Utility", "GetStates").then(function(statesArray) {
				stateStore = new Memory({data:statesArray});
			});

			var visitsPromise = serverRequest("Patient.Activity.Utility", "GetPatientVisits", {"ndocNum": ndocNum}).then(function(visitsArray) {
				visitStore = new Memory({data:visitsArray});
				visitStore.setData(visitStore.query({},{sort:[{attribute:"sort",descending:true}]}));
			});

			var referralPromise = serverRequest("Patient.Activity.Utility", "GetReferredDisciplines", {"ndocNum": ndocNum}).then(function(results) {
				referralArray = results;
			});

			var usersPromise = serverRequest("Patient.Activity.Utility", "GetUsers").then(function(userArray) {
				userStore = new Memory({data:userArray});
			});

			var constantsPromise = serverRequest("Patient.Activity.Utility", "EditorConstants", {"ndocNum": ndocNum}).then(function(response) {
				payrollOn = parseInt(response.payrollOn);
				supervisoryAccess = parseInt(response.supervisoryAccess);
				patientName = response.patientName;
				dischargeDateString = response.dischargeDateString;
				costCenterIdList = response.costCenterIdList;
				requireSummary = response.requireSummary;

				dischargeDate = (dischargeDateString !== "" ? locale.parse(dischargeDateString,{selector:"date",datePattern:"yyyy-MM-dd"}) : null);

				costCenterIdArray = (costCenterIdList).split(",");
				if (costCenterIdArray.length > 0) {
					costCenterIdArray.push(999);
				}
			});

			var tableManagerPromise = TableManager.load(["Tables.PatientActivityType", "Tables.PatientActivityActionTaken", "Tables.PatientActivityFromTo", "Tables.PatientActivityLocation", "Tables.PatientActivityReason", "Tables.OnCallPriority", "Tables.PayrollPaycode", "Tables.PayrollCostCenter", "Tables.Discipline", "Tables.PatientCareMeetingType"]).then(function() {
				activityTypeStore	= TableManager.get("Tables.PatientActivityType");
				actionTakenStore	= TableManager.get("Tables.PatientActivityActionTaken");
				fromToStore			= TableManager.get("Tables.PatientActivityFromTo");
				locationStore		= TableManager.get("Tables.PatientActivityLocation");
				reasonStore			= TableManager.get("Tables.PatientActivityReason");
				priorityStore		= TableManager.get("Tables.OnCallPriority");
				paycodeStore		= TableManager.get("Tables.PayrollPaycode");
				costCenterStore		= TableManager.get("Tables.PayrollCostCenter");
				disciplineStore		= TableManager.get("Tables.Discipline");
				meetingTypeStore	= TableManager.get("Tables.PatientCareMeetingType");

				arrayUtils.forEach(meetingTypeStore.data, function(type) {
					type.reasonIdArray = type.reasonIdList.toString().split(",");
				});

				paycodeStore.put({id:999,name:"UNKNOWN",active:"Yes"});
				costCenterStore.put({id:999,name:"UNKNOWN",active:"Yes"});
				
				// Remove inactive paycodes
				arrayUtils.forEach(reasonStore.data,function(reason) {
					if (reason.paycodeIds !== "") {
						var paycodeIdArray = reason.paycodeIds.toString().split(",");
						var paycodeIdsToKeep = [];
						arrayUtils.forEach(paycodeIdArray,function(paycodeId) {
							var paycode = paycodeStore.get(paycodeId);
							if (paycode && paycode.active === "Yes") {
								paycodeIdsToKeep.push(paycodeId);
							}
						});
						reason.paycodeIds = paycodeIdsToKeep.join(",");
					}
				});
			});

			var storesReadyPromise = new Deferred();
			all([activitiesPromise, statesPromise, visitsPromise, referralPromise, usersPromise, constantsPromise, tableManagerPromise]).then(function() {
				var activityArray = [];
				arrayUtils.forEach(activityList,function(activity) {
					activityArray.push(Activity.getActivity(activity));
				});
				activityStore = new Memory({data:activityArray});
				storesReadyPromise.resolve();
			});

			return storesReadyPromise;
		};
		
		return ActivityEditor;
	}
);